# daily-email-lambda
import json
import boto3
import datetime as dt 

SNS_ARN = 'arn:aws:sns:eu-central-1:639276492473:send-data-sns-topic'   # ARN teme 'send-data-sns-topic'
BUCKET_NAME = 'rlovric-lambda-bucket'


def get_rate(data, i, currency):
    # funkcija koja vraća i-tu float vrijednost iz json argumenta, u valuti definiranoj u argumentu
    return data[i]['bpi'][currency]['rate_float']

def generate_filenames(today):
    # funkcija koja generira imena datoteka ovisno o datetime argumentu
    yesterday = today - dt.timedelta(days=1)
    name_list = []
    
    for hour in range(yesterday.hour, 24):
        name_list.append(f'{yesterday.year}-{yesterday.month}-{yesterday.day}-{hour}.json')
    for hour in range(0, today.hour+1):
        name_list.append(f'{today.year}-{today.month}-{today.day}-{hour}.json')
    
    return name_list

def lambda_handler(event, context):
    
    today = dt.datetime.now()
    # generiranje liste imena datoteka
    filenames_list = generate_filenames(today)

    # instanciranje S3 resursa iz boto3 biblioteke
    s3 = boto3.resource('s3')

    object_list = []

    for name in filenames_list:
        # instanciranje S3 objekta (datoteka), u određenom bucket-u pod određenim imenom
        object = s3.Object(BUCKET_NAME, name)
        try:
            # čitanje datoteke i spremanje u 'object_list' listu json objekata
            object_bytes = object.get()['Body'].read()
            object_data = object_bytes.decode('utf8')
            object_list.append(json.loads(object_data))
            print(f'Read successfuly file: {name}')
        except Exception as e:
            print(f'Error reading file: {name}\nDetail: {str(e)}')

    # izvlačenje trenutne cijene pomoću get_rate() funkcije, koja se nalazi na zadnjem indeksu u object_list listi
    current_price = {
        'eur': get_rate(object_list, -1, 'EUR'),
        'usd': get_rate(object_list, -1, 'USD')
    }
    
    # računanje apsolutne i relativne greške u rasponu od jednog sata i jednodnevnom rasponu
    hour_diff = get_rate(object_list, -1, 'EUR') - get_rate(object_list, -2, 'EUR')
    day_diff = get_rate(object_list, -1, 'EUR') - get_rate(object_list, 0, 'EUR')
    hour_percentage = hour_diff / abs(get_rate(object_list, -2, 'EUR')) * 100
    day_percentage = day_diff / abs(get_rate(object_list, -0, 'EUR')) * 100

    # definiranje stringova i ubacivanje prethodno dobivenih vrijednosi
    first_string = f'Trenutna cijena Bitcoina je: € {current_price["eur"]:.2f} / $ {current_price["usd"]:.2f}\n\n'
    hour_string = f'U proteklih sat vremena, cijena se promjenila € {hour_diff:.2f}, {hour_percentage:.2f}%\n'
    day_string = f'U proteklih 24 sata, cijena se promjenila € {day_diff:.2f}, {day_percentage:.2f}%'

    # instanciranje SNS klijenta iz boto3 biblioteke
    sns_client = boto3.client('sns')
    # objavljivanje poruke na SNS temu
    # TopicArn je ARN naše SNS teme
    # Subject je Predmet
    # Message je obavijest koja se šalje, u ovom slučaju su prethodno definirani stringovi
    sns_client.publish(
        TopicArn = SNS_ARN,
        Subject = 'Dnevna cijena',
        Message = first_string + hour_string + day_string
    )

    return {'body': 'Mail sent successfuly', 'status_code': 200}
